/*
 * *************************************************
 * Copyright (c) 2018 QMTH. All Rights Reserved.
 * Created by Deason on 2018-07-20 14:15:02.
 * *************************************************
 */

package cn.com.qmth.examcloud.app.core.config;

import cn.com.qmth.examcloud.app.model.DeviceRecord;
import cn.com.qmth.examcloud.app.model.LoginInfo;
import cn.com.qmth.examcloud.app.model.Result;
import cn.com.qmth.examcloud.app.model.User;
import cn.com.qmth.examcloud.app.service.DeviceRecordService;
import cn.com.qmth.examcloud.app.service.UserAuthService;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.servlet.handler.HandlerInterceptorAdapter;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.Date;
import java.util.Map;

import static cn.com.qmth.examcloud.app.model.Constants.PARAM_PASSWORD;
import static cn.com.qmth.examcloud.app.model.Constants.PLATFORM_SESSION_EXPIRE_TIME;

/**
 * API请求拦截器
 *
 * @author: fengdesheng
 * @since: 2018/7/16
 */
public class AccessInterceptor extends HandlerInterceptorAdapter {
    private final static Logger log = LoggerFactory.getLogger(AccessInterceptor.class);
    @Autowired
    private DeviceRecordService deviceRecordService;
    @Autowired
    private UserAuthService userAuthService;

    @Override
    public boolean preHandle(HttpServletRequest request, HttpServletResponse response, Object handler) throws Exception {
        DeviceRecord record = this.parseParams(request);
        //用户带的login token实际为缓存的redisKey
        /*String redisKey = record.getLoginToken();
        if (StringUtils.isNotBlank(redisKey)) {
            //通过redisKey获取原始的登录信息，loginInfo为null则代表尚未登录
            LoginInfo loginInfo = userAuthService.getLoginInfo(redisKey);
            if (loginInfo != null) {
                //判断原始的login token是否在有效时间内
                if (loginInfo.hasExpired(PLATFORM_SESSION_EXPIRE_TIME)) {
                    //已超过有效时间，则自动登录续期
                    Result<User> result = userAuthService.login(loginInfo);
                    User userInfo = result.getData();
                    if (userInfo != null) {
                        //登录成功后缓存新的login token
                        log.debug("redisKey:" + redisKey + " newLoginToken:" + userInfo.getToken());
                        loginInfo.setToken(userInfo.getToken());
                        loginInfo.setCreateTime(new Date());
                        userAuthService.cacheLoginInfo(loginInfo, redisKey);
                    }
                }
                //将login token绑定到request请求中
                request.setAttribute("loginToken", loginInfo.getToken());
            }
            //初始化内部接口请求鉴权
            userAuthService.initRequestTrace();
        }*/
        //异步保存设备访问记录
        deviceRecordService.addDeviceRecord(record);
        return true;
    }

    private DeviceRecord parseParams(HttpServletRequest request) {
        String params = this.getParams(request);
        String url = request.getServletPath() + params;
        String ip = this.getIp(request);
        //封装设备访问记录信息
        DeviceRecord record = new DeviceRecord();
        record.setSystem(request.getHeader("system"));
        record.setDeviceId(request.getHeader("deviceId"));
        record.setNetType(request.getHeader("netType"));
        record.setBrand(request.getHeader("brand"));
        record.setModel(request.getHeader("model"));
        record.setSysVersion(request.getHeader("sysVersion"));
        record.setAppVersion(request.getHeader("appVersion"));
        record.setAppCode(request.getHeader("appCode"));
        record.setPatchCode(request.getHeader("patchCode"));
        record.setAccount(request.getHeader("account"));
        record.setLoginKey(request.getHeader("key"));
        record.setLoginToken(request.getHeader("token"));
        record.setUrl(url);
        record.setIp(ip);
        record.setCreateDate(new Date());
        return record;
    }

    private String getParams(HttpServletRequest request) {
        StringBuilder params = new StringBuilder();
        Map<String, String[]> map = request.getParameterMap();
        for (Map.Entry<String, String[]> entry : map.entrySet()) {
            String name = entry.getKey();
            String[] values = entry.getValue();
            params.append("&").append(name).append("=");
            if (values.length > 0) {
                if (PARAM_PASSWORD.equalsIgnoreCase(name)) {
                    params.append("***");
                } else {
                    params.append(values[0]);
                }
            }
        }
        /*try {
            //将RequestBody中的参数转为JSON格式(注意：request.getInputStream()执行一次后则导致Controller再无法取到RequestBody值)
            byte[] requestBody = StreamUtils.copyToByteArray(request.getInputStream());
            JSONObject object = JSON.parseObject((new String(requestBody, "UTF-8")));
            if (object != null) {
                System.out.println(object.toJSONString());
            }
        } catch (Exception e) {
            e.printStackTrace();
        }*/
        return params.toString().replaceFirst("&", "?");
    }

    private String getIp(HttpServletRequest request) {
        String ip = request.getHeader("X-Forwarded-For");
        if (StringUtils.isEmpty(ip) || "unknown".equalsIgnoreCase(ip)) {
            ip = request.getHeader("Proxy-Client-IP");
        }
        if (StringUtils.isEmpty(ip) || "unknown".equalsIgnoreCase(ip)) {
            ip = request.getHeader("WL-Proxy-Client-IP");
        }
        if (StringUtils.isEmpty(ip) || "unknown".equalsIgnoreCase(ip)) {
            ip = request.getHeader("HTTP_CLIENT_IP");
        }
        if (StringUtils.isEmpty(ip) || "unknown".equalsIgnoreCase(ip)) {
            ip = request.getHeader("HTTP_X_FORWARDED_FOR");
        }
        if (StringUtils.isEmpty(ip) || "unknown".equalsIgnoreCase(ip)) {
            ip = request.getRemoteAddr();
        }
        return ip;
    }

}